@@ -91,7 +91,7 @@ class Rediz(RedizConventions):
     def get_predictions(self, name, delay=None, delays=None):
         return self._get_predictions_implementation(name=name, delay=delay, delays=delays)
 
-    def get_cdf(self, name, delay=None, values=None, top=10, min_balance=-500):
+    def get_cdf(self, name, delay=None, values=None, top=10, min_balance=-5000):
         """
         :param values:   Abscissa for CDF
         :param top:      Number of top participants to use
@@ -200,22 +200,51 @@ class Rediz(RedizConventions):
 
     def get_previous_monthly_overall_leaderboard(self):
         last_month_day = datetime.datetime.now().replace(day=1) - datetime.timedelta(days=2)
-        return self._get_custom_leaderboard_implementation(sponsor=None,dt=last_month_day,count=200)
+        return self._get_custom_leaderboard_implementation(sponsor_code=None, dt=last_month_day, count=200)
 
     def get_monthly_overall_leaderboard(self):
-        return self._get_custom_leaderboard_implementation(sponsor=None,dt=datetime.datetime.now(),count=200)
+        return self._get_custom_leaderboard_implementation(sponsor_code=None, dt=datetime.datetime.now(), count=200)
 
     # By sponsor ...
+    # Allow user to pass either a write_key or a write_code (recall code = shash(key) )
 
-    def get_sponsored_leaderboard(self, sponsor):
-        return self._get_custom_leaderboard_implementation(sponsor=sponsor,dt=None,count=200)
+    def code_from_code_or_key(self, sponsor):
+        if self.is_valid_key(sponsor):
+            return self.shash(sponsor)
+        elif self.animal_from_code(sponsor):
+            return sponsor
+
+    def get_regular_monthly_sponsored_leaderboard(self, sponsor):
+        """ Excludes z's """
+        code = self.code_from_code_or_key(sponsor)
+        if code:
+            return self._get_custom_leaderboard_implementation(sponsor_code=code, dt=None, count=200, name='mystream')
+
+    def get_zscore_monthly_sponsored_leaderboard(self, sponsor):
+        code = self.code_from_code_or_key(sponsor)
+        if code:
+            return self._get_custom_leaderboard_implementation(sponsor_code=code, dt=None, count=200, name='z1~')
+
+    def get_bivariate_monthly_sponsored_leaderboard(self, sponsor):
+        code = self.code_from_code_or_key(sponsor)
+        if code:
+            return self._get_custom_leaderboard_implementation(sponsor_code=code, dt=None, count=200, name='z2~')
+
+    def get_trivariate_monthly_sponsored_leaderboard(self, sponsor):
+        code = self.code_from_code_or_key(sponsor)
+        if code:
+            return self._get_custom_leaderboard_implementation(sponsor_code=code, dt=None, count=200, name='z3~')
 
     def get_monthly_sponsored_leaderboard(self, sponsor):
-        return self._get_custom_leaderboard_implementation(sponsor=sponsor,dt=datetime.datetime.now(),count=200)
+        code = self.code_from_code_or_key(sponsor)
+        if code:
+            return self._get_custom_leaderboard_implementation(sponsor_code=code, dt=datetime.datetime.now(), count=200)
 
     def get_previous_monthly_sponsored_leaderboard(self, sponsor):
-        last_month_day = datetime.datetime.now().replace(day=1) - datetime.timedelta(days=2)
-        return self._get_custom_leaderboard_implementation(sponsor=sponsor, dt=last_month_day,count=200)
+        code = self.code_from_code_or_key(sponsor)
+        if code:
+            last_month_day = datetime.datetime.now().replace(day=1) - datetime.timedelta(days=2)
+            return self._get_custom_leaderboard_implementation(sponsor_code=code, dt=last_month_day, count=200)
 
 
 
@@ -550,6 +579,7 @@ class Rediz(RedizConventions):
 
         if ndxs:
             exists_pipe = self.client.pipeline(transaction=False)
+            fakeredis = self.client.connection is None
             for name in names:
                 exists_pipe.hexists(name=self._ownership_name(),key=name)
             exists = exists_pipe.execute()
@@ -561,7 +591,7 @@ class Rediz(RedizConventions):
                         rejected.append({"ndx":ndx,"name":name,"write_key":None,"errror":"invalid write_key"})
                     else:
                         ttl = self._cost_based_ttl(value=value, budget=budget)
-                        new_pipe, intent = self._new_page(new_pipe,ndx=ndx, name=name,value=value,write_key=write_key, budget=budget)
+                        new_pipe, intent = self._new_page(new_pipe,ndx=ndx, name=name,value=value,write_key=write_key, budget=budget, fakeredis=fakeredis)
                         executed.append(intent)
                 else:
                     ignored_ndxs.append(ndx)
@@ -630,13 +660,13 @@ class Rediz(RedizConventions):
 
         return executed
 
-    def _new_obscure_page( self, pipe, ndx, name, value, write_key, budget):
+    def _new_obscure_page( self, pipe, ndx, name, value, write_key, budget, fakeredis=False):
         """ Almost the same as a new page """
-        pipe, intent = self._new_page( pipe=pipe, ndx=ndx, name=name, value=value, write_key=write_key, budget=budget )
+        pipe, intent = self._new_page( pipe=pipe, ndx=ndx, name=name, value=value, write_key=write_key, budget=budget, fakeredis=fakeredis )
         intent.update({"obscure":True})
         return pipe, intent
 
-    def _new_page( self, pipe, ndx, name, value, write_key, budget ):
+    def _new_page( self, pipe, ndx, name, value, write_key, budget, fakeredis=False ):
         """ Create new page:
               pipe         :  Redis pipeline that will be modified
             Returns also:
@@ -645,6 +675,18 @@ class Rediz(RedizConventions):
         # Establish ownership
         pipe.hset(name=self._ownership_name(),key=name,value=write_key)
         pipe.sadd(self._NAMES, name)
+        # Charge for stream creation
+        create_charge = budget*self._CREATE_COST
+        transaction_record = {"settlement_time": str(datetime.datetime.now()), "type": "create",
+                              "write_key": write_key, "amount":create_charge,"message":"charge for new stream creation","name":name}
+        pipe.hincrbyfloat(name=self._BALANCES, key=write_key, amount=create_charge)
+        if not fakeredis:
+            log_names = [self.transactions_name(),
+                         self.transactions_name(write_key=write_key),
+                         self.transactions_name(write_key=write_key, name=name)]
+            for ln in log_names:
+                pipe.xadd(name=ln, fields=transaction_record, maxlen=self.TRANSACTIONS_LIMIT)
+                pipe.expire(name=ln, time=self._TRANSACTIONS_TTL)
         # Then modify
         pipe, intent = self._modify_page(pipe=pipe,ndx=ndx,name=name,value=value,budget=budget)
         intent.update({"new":True,"write_key":write_key,"value":value})
@@ -1344,10 +1386,10 @@ class Rediz(RedizConventions):
                             # from more than one contributor, hopefully leading to more accurate percentiles
                             percentile_scenarios = list()
                             for window_ndx in range(num_windows):
-                                if len(percentile_scenarios) < 5:
+                                if len(percentile_scenarios) < 10:
                                     _ndx = scenarios_lookup[name][delay_ndx][window_ndx]
                                     percentile_scenarios = retrieved[_ndx]
-                                    some_percentiles = True
+                                    some_percentiles = len(percentile_scenarios)>0
                             percentiles[name][delay_ndx] = self._zmean_scenarios_percentile(percentile_scenarios=percentile_scenarios)
 
         # ---- Rewards and leaderboard update pipeline
@@ -1378,9 +1420,12 @@ class Rediz(RedizConventions):
                                              self.leaderboard_name(name=name),
                                              self.leaderboard_name(name=None, delay=delay),
                                              self.leaderboard_name(name=name, delay=delay),
-                                             self.custom_leaderboard_name(sponsor=sponsor, name=name, dt=datetime.datetime.now()),
-                                             self.custom_leaderboard_name(sponsor=sponsor, name=name),
-                                             self.custom_leaderboard_name(sponsor=sponsor, dt=datetime.datetime.now())
+                                             self.custom_leaderboard_name(sponsor=None, name=None),  # Overall all time
+                                             self.custom_leaderboard_name(sponsor=sponsor, name=None), # Sponsor category
+                                             self.custom_leaderboard_name(sponsor=sponsor, name=name),  # Sponsor and cateogry
+                                             self.custom_leaderboard_name(sponsor=sponsor, dt=datetime.datetime.now()), # Sponsor and month
+                                             self.custom_leaderboard_name(sponsor=sponsor, name=name, dt=datetime.datetime.now()), # Sponsor and category and month
+                                             self.custom_leaderboard_name(sponsor=sponsor, name=name, dt=datetime.datetime.now()), # Sponsor this month
                                              ]
                         for (recipient, amount) in payments.items():
                             # Record keeping
@@ -1400,8 +1445,8 @@ class Rediz(RedizConventions):
                                                   "stream": name,
                                                   "delay":delay,
                                                   "value":value,
-                                                  "submitted":pool,
-                                                  "rewarded":num_rewarded,
+                                                  "submissions_count":pool,
+                                                  "submissions_close":num_rewarded,
                                                   "stream_owner_code":write_code,
                                                   "recipient_code":recipient_code}
                             log_names = [ self.transactions_name(),
@@ -1424,26 +1469,28 @@ class Rediz(RedizConventions):
             z_names   = list()
             z_curves  = list()
             z_write_keys = list()
-            for delay_ndx, delay in enumerate(self.DELAYS):
+            for delay_ndx, delay in enumerate(self.ZDELAYS):
                 percentiles1 = [ percentiles[name][delay_ndx] for name in names ]
-                num_names = len(names)
-                selections = list(itertools.combinations(list(range(num_names)), 1))
-                if with_copulas and num_names<=10:
-                    selections2 = list(itertools.combinations(list(range(num_names)), 2))
-                    selections3 = list(itertools.combinations(list(range(num_names)), 3))
-                    selections = selections + selections2 + selections3
+                legit = not( all( [ abs(p1-0.5)<1e-5 for p1 in percentiles1 ] ))
+                if legit:
+                    num_names = len(names)
+                    selections = list(itertools.combinations(list(range(num_names)), 1))
+                    if with_copulas and num_names<=10:
+                        selections2 = list(itertools.combinations(list(range(num_names)), 2))
+                        selections3 = list(itertools.combinations(list(range(num_names)), 3))
+                        selections = selections + selections2 + selections3
 
-                for selection in selections:
-                    selected_names   = [ names[o] for o in selection ]
-                    dim              = len(selection)
-                    z_budget         = sum( [ budgets[o] for o in selection ] ) / (dim**3)   # FIXME: Why int?
-                    selected_prctls  = [ percentiles1[o] for o in selection ]
-                    zcurve_value     = self.to_zcurve(selected_prctls)
-                    zname            = self.zcurve_name(selected_names, delay)
-                    z_names.append(zname)
-                    z_budgets.append(z_budget)
-                    z_curves.append(zcurve_value)
-                    z_write_keys.append(first_write_key)
+                    for selection in selections:
+                        selected_names   = [ names[o] for o in selection ]
+                        dim              = len(selection)
+                        z_budget         = sum( [ budgets[o] for o in selection ] ) / (dim**3)   # FIXME: Why int?
+                        selected_prctls  = [ percentiles1[o] for o in selection ]
+                        zcurve_value     = self.to_zcurve(selected_prctls)
+                        zname            = self.zcurve_name(selected_names, delay)
+                        z_names.append(zname)
+                        z_budgets.append(z_budget)
+                        z_curves.append(zcurve_value)
+                        z_write_keys.append(first_write_key)
             if z_names:
                 self._mset_implementation(budgets=z_budgets, names=z_names, values=z_curves, write_keys=z_write_keys, with_percentiles=False, with_copulas=False)
         return percentiles
@@ -1708,8 +1755,8 @@ class Rediz(RedizConventions):
         leaderboard = list(reversed(self.client.zrange(name=pname, start=-count, end=-1, withscores=True)))
         return OrderedDict([(self.animal_from_code(code),score) for code,score in leaderboard]) if readable else dict(leaderboard)
 
-    def _get_custom_leaderboard_implementation(self, sponsor, dt, count, readable=True):
-        pname = self.custom_leaderboard_name(sponsor=sponsor,dt=dt)
+    def _get_custom_leaderboard_implementation(self, sponsor_code, dt, count, readable=True, name=None):
+        pname = self.custom_leaderboard_name(sponsor=sponsor_code, dt=dt, name=name)
         leaderboard = list(reversed(self.client.zrange(name=pname, start=-count, end=-1, withscores=True)))
         return OrderedDict([(self.animal_from_code(code), score) for code, score in leaderboard]) if readable else dict(
             leaderboard)
